@@ -1,24 +1,24 @@
 /*
  * ggcov - A GTK frontend for exploring gcov coverage data
- * Copyright (c) 2005 Greg Banks <gnb@users.sourceforge.net>
+ * Copyright (c) 2005-2020 Greg Banks <gnb@fastmail.fm>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "common.h"
-#include "cov.H"
+#include "cov_priv.H"
 #include "filename.h"
 #include "estring.H"
 #include "tok.H"
@@ -27,11 +27,10 @@
 #include "php_scenegen.H"
 #include "callgraph_diagram.H"
 #include "lego_diagram.H"
-#include "fakepopt.h"
+#include "argparse.H"
+#include "logging.H"
 #include <db.h>
 
-CVSID("$Id: ggcov-webdb.c,v 1.11 2010-05-09 05:37:15 gnb Exp $");
-
 #define V(major,minor,patch)    ((major)*10000+(minor)*1000+(patch))
 #define DB_VERSION_CODE V(DB_VERSION_MAJOR,DB_VERSION_MINOR,DB_VERSION_PATCH)
 
@@ -48,15 +47,12 @@ CVSID("$Id: ggcov-webdb.c,v 1.11 2010-05-09 05:37:15 gnb Exp $");
 #endif
 
 char *argv0;
-static list_t<const char> files;	    /* incoming specification from commandline */
-
-static char *dump_mode = NULL;
-static const char *output_tarball = "ggcov.webdb.tgz";
 
 static hashtable_t<void, unsigned int> *file_index;
 static hashtable_t<void, unsigned int> *function_index;
 static hashtable_t<void, unsigned int> *callnode_index;
 static list_t<cov_function_t> *all_functions;
+static logging::logger_t &_log = logging::find_logger("web");
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
@@ -136,7 +132,7 @@ systemf(const char *fmt, ...)
     cmd.append_vprintf(fmt, args);
     va_end(args);
 
-    dprintf1(D_WEB, "Running \"%s\"\n", cmd.data());
+    _log.debug("Running \"%s\"\n", cmd.data());
     return system(cmd.data());
 }
 
@@ -166,7 +162,7 @@ save_file_lines(DB *db, cov_file_t *f)
 	ser.end_array();
     }
     ser.end_array();
-    
+
     // Store the serialised line array
     string_var key = g_strdup_printf("FL%u", ftag(f));
     if ((ret = db->put(db, 0, dbt(key), dbt(ser), 0)))
@@ -222,30 +218,12 @@ save_filename_index(DB *db)
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
-// like cov_function_t::list_all() but includes suppressed
-// functions as well.  TODO: is that a good idea?
-
-static list_t<cov_function_t> *
-really_list_all_functions(void)
-{
-    list_t<cov_function_t> *all = new list_t<cov_function_t>;
-
-    for (list_iterator_t<cov_file_t> iter = cov_file_t::first() ; *iter ; ++iter)
-    {
-	for (ptrarray_iterator_t<cov_function_t> fnitr = (*iter)->functions().first() ; *fnitr ; ++fnitr)
-	    all->append(*fnitr);
-    }
-
-    all->sort(cov_function_t::compare);
-    return all;
-}
-
 static void
 build_global_function_index(void)
 {
     unsigned int n = 0;
 
-    all_functions = really_list_all_functions();
+    all_functions = cov_list_all_functions();
 
     function_index = new hashtable_t<void, unsigned int>;
     for (list_iterator_t<cov_function_t> iter = all_functions->first() ; *iter ; ++iter)
@@ -272,7 +250,7 @@ save_global_function_index(DB *db)
     unique = new hashtable_t<const char, list_t<cov_function_t> >;
     for (list_iterator_t<cov_function_t> fniter = all_functions->first() ; *fniter ; ++fniter)
     {
-    	cov_function_t *fn = *fniter;
+	cov_function_t *fn = *fniter;
 	if (fn->is_suppressed())
 	    continue;
 	list_t<cov_function_t> *list = unique->lookup(fn->name());
@@ -283,7 +261,7 @@ save_global_function_index(DB *db)
 	}
 	list->append(fn);
     }
-    
+
     // PHP-serialise the function index
     unique->keys(&keys);
     ser.begin_array(unique->size());
@@ -343,10 +321,10 @@ save_global_function_list(DB *db)
 	if (fn->is_suppressed())
 	    continue;
 
-    	label.truncate();
+	label.truncate();
 	label.append_string(fn->name());
 
-    	/* see if we need to present some more scope to uniquify the name */
+	/* see if we need to present some more scope to uniquify the name */
 	list_iterator_t<cov_function_t> next = iter.peek_next();
 	list_iterator_t<cov_function_t> prev = iter.peek_prev();
 	if ((*next && !strcmp((*next)->name(), fn->name())) ||
@@ -356,7 +334,7 @@ save_global_function_list(DB *db)
 	    label.append_string(fn->file()->minimal_name());
 	    label.append_string("]");
 	}
-	
+
 	ser.string(label);
 	ser.string(fn->file()->minimal_name());
     }
@@ -375,7 +353,7 @@ save_file_function_indexes(DB *db)
 {
     for (list_iterator_t<cov_file_t> iter = cov_file_t::first() ; *iter ; ++iter)
     {
-    	cov_file_t *f = *iter;
+	cov_file_t *f = *iter;
 	php_serializer_t ser;
 	int ret;
 
@@ -471,7 +449,7 @@ save_one_summary_f(DB *db, cov_scope_t *sc, const char *key)
     ser.next_key();
     serialise_ulong_array(&ser, cov::NUM_STATUS, stats->branches_by_status());
     ser.end_array();
-    
+
     if ((ret = db->put(db, 0, dbt(key), dbt(ser), 0)))
     {
 	db->err(db, ret, "save_one_summary_f");
@@ -502,7 +480,7 @@ save_summaries(DB *db)
     // Save a function scope object for each function
     for (list_iterator_t<cov_function_t> fniter = all_functions->first() ; *fniter ; ++fniter)
     {
-    	cov_function_t *fn = *fniter;
+	cov_function_t *fn = *fniter;
 	sc = new cov_function_scope_t(fn);
 	string_var key = g_strdup_printf("US%u", fntag(fn));
 	save_one_summary_f(db, sc, key);
@@ -518,18 +496,15 @@ build_callnode_index(void)
     unsigned int n = 0;
 
     callnode_index = new hashtable_t<void, unsigned int>;
-    for (cov_callnode_iter_t cnitr = cov_callnode_t::first() ; *cnitr ; ++cnitr)
-	callnode_index->insert((void *)*cnitr, new unsigned int(++n));
+    for (cov_callspace_iter_t csitr = cov_callgraph.first() ; *csitr ; ++csitr)
+    {
+	for (cov_callnode_iter_t cnitr = (*csitr)->first() ; *cnitr ; ++cnitr)
+	    callnode_index->insert((void *)*cnitr, new unsigned int(++n));
+    }
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
-static int
-compare_node_by_name(const cov_callnode_t *cn1, const cov_callnode_t *cn2)
-{
-    return strcmp(cn1->name, cn2->name);
-}
-
 static void
 save_callnode_index(DB *db)
 {
@@ -538,9 +513,12 @@ save_callnode_index(DB *db)
     int ret;
 
     // Sort the callnode index, for the node <select>
-    for (cov_callnode_iter_t cnitr = cov_callnode_t::first() ; *cnitr ; ++cnitr)
-	all.append(*cnitr);
-    all.sort(compare_node_by_name);
+    for (cov_callspace_iter_t csitr = cov_callgraph.first() ; *csitr ; ++csitr)
+    {
+	for (cov_callnode_iter_t cnitr = (*csitr)->first() ; *cnitr ; ++cnitr)
+	    all.append(*cnitr);
+    }
+    all.sort(cov_callnode_t::compare_by_name_and_file);
 
     // PHP-serialise the callnode index
     ser.begin_array(callnode_index->size());
@@ -565,33 +543,12 @@ save_callnode_index(DB *db)
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
-static int
-compare_in_arc_by_count(const cov_callarc_t *ca1, const cov_callarc_t *ca2)
-{
-    int r;
-
-    r = -u64cmp(ca1->count, ca2->count);
-    if (!r)
-	r = strcmp(ca1->from->name, ca2->from->name);
-    return r;
-}
-
-static int
-compare_out_arc_by_count(const cov_callarc_t *ca1, const cov_callarc_t *ca2)
-{
-    int r;
-
-    r = -u64cmp(ca1->count, ca2->count);
-    if (!r)
-	r = strcmp(ca1->to->name, ca2->to->name);
-    return r;
-}
-
 static void
 serialize_arc_list(php_serializer_t *ser, list_t<cov_callarc_t> &arcs, gboolean out)
 {
     list_t<cov_callarc_t> copy = arcs.copy();
-    copy.sort(out ? compare_out_arc_by_count : compare_in_arc_by_count);
+    copy.sort(out ? cov_callarc_t::compare_by_count_and_to :
+		    cov_callarc_t::compare_by_count_and_from);
 
     ser->begin_array(copy.length());
     for (list_iterator_t<cov_callarc_t> itr = copy.first() ; *itr ; ++itr)
@@ -636,8 +593,11 @@ save_callnode(cov_callnode_t *cn, DB *db)
 static void
 save_callgraph(DB *db)
 {
-    for (cov_callnode_iter_t cnitr = cov_callnode_t::first() ; *cnitr ; ++cnitr)
-	save_callnode(*cnitr, db);
+    for (cov_callspace_iter_t csitr = cov_callgraph.first() ; *csitr ; ++csitr)
+    {
+	for (cov_callnode_iter_t cnitr = (*csitr)->first() ; *cnitr ; ++cnitr)
+	    save_callnode(*cnitr, db);
+    }
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
@@ -688,12 +648,12 @@ save_reports(DB *db)
 			       (char *)0);
     if ((fd = mkstemp((char *)tmp_filename.data())) < 0)
     {
-	perror(tmp_filename);
+	_log.perror(tmp_filename);
 	exit(1);
     }
     if ((fp = fdopen(fd, "r+")) == NULL)
     {
-	perror("fdopen");
+	_log.perror("fdopen");
 	close(fd);
 	exit(1);
     }
@@ -708,7 +668,7 @@ save_reports(DB *db)
 	rewind(fp);
 	if (ftruncate(fd, (off_t)0) < 0)
 	{
-	    perror(tmp_filename);
+	    _log.perror(tmp_filename);
 	    exit(1);
 	}
 
@@ -719,7 +679,7 @@ save_reports(DB *db)
 	// Read the report output into buffer
 	if ((len = fd_length(fd)) < 0)
 	{
-	    perror(tmp_filename);
+	    _log.perror(tmp_filename);
 	    exit(1);
 	}
 	buffer.truncate_to(len);
@@ -727,14 +687,13 @@ save_reports(DB *db)
 	int r = fread((char *)buffer.data(), 1, len, fp);
 	if (r != len)
 	{
-	    perror(tmp_filename);
-	    fprintf(stderr, "len=%d r=%d\n", len, r);
+	    _log.perror(tmp_filename);
 	    exit(1);
 	}
 
 	// PHP-serialise the report data
 	ser.stringl(buffer.data(), buffer.length());
-	
+
 	// Save the report data
 	key = g_strdup_printf("R%u", n);
 	if ((ret = db->put(db, 0, dbt(key), dbt(ser), 0)))
@@ -812,7 +771,7 @@ save_diagrams(DB *db)
 	di->get_bounds(&bounds);
 	sg->bounds(bounds.x1, bounds.y1,
 		   (bounds.x2 - bounds.x1), (bounds.y2 - bounds.y1));
-	
+
 	// Save the diagram data
 	key = g_strdup_printf("G%u", i);
 	if ((ret = db->put(db, 0, dbt(key), dbt(sg->data()), 0)))
@@ -836,8 +795,9 @@ create_source_symlinks(const char *tempdir)
 	string_var link = g_strconcat(tempdir, "/", f->minimal_name(), (char *)0);
 	string_var tempmindir = file_dirname(link);
 	file_build_tree(tempmindir, 0755);
-	dprintf2(D_WEB, "symlink %s -> %s\n", link.data(), f->name());
-	symlink(f->name(), link);
+	_log.debug("symlink %s -> %s\n", link.data(), f->name());
+	if (symlink(f->name(), link) < 0)
+	    perror(f->name());
     }
 }
 
@@ -852,85 +812,53 @@ create_source_symlinks(const char *tempdir)
  * parse arguments in a way which avoids potentially buggy duplicate
  * specification of options, i.e. we simulate popt in fakepopt.c!
  */
-static poptContext popt_context;
-static struct poptOption popt_options[] =
+
+class webdb_params_t : public cov_project_params_t
 {
+public:
+    webdb_params_t();
+    ~webdb_params_t();
+
+    ARGPARSE_STRING_PROPERTY(output_tarball);
+    ARGPARSE_STRING_PROPERTY(dump_mode);
+
+protected:
+    void setup_parser(argparse::parser_t &parser)
     {
-    	"output-file",    	    	    	/* longname */
-	'f',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&output_tarball,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"name of the output (in .tgz format), or - for stdout", /* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    {
-    	"dump",					/* longname */
-	'\0',  	    	    	    	    	/* shortname */
-	POPT_ARG_STRING,  	    	    	/* argInfo */
-	&dump_mode,     	    	    	/* arg */
-	0,  	    	    	    	    	/* val 0=don't return */
-	"dump the entire database",		/* descrip */
-	0	    	    	    	    	/* argDescrip */
-    },
-    COV_POPT_OPTIONS
-    POPT_AUTOHELP
-    POPT_TABLEEND
+	cov_project_params_t::setup_parser(parser);
+	parser.add_option('f', "output-file")
+	      .description("name of the output (in .tgz format), or - for stdout")
+	      .setter((argparse::arg_setter_t)&webdb_params_t::set_output_tarball);
+	parser.add_option('\0', "dump")
+	      .description("dump the entire database")
+	      .setter((argparse::arg_setter_t)&webdb_params_t::set_dump_mode);
+	parser.set_other_option_help("[OPTIONS] [executable|source|directory]...");
+    }
 };
 
-static void
-parse_args(int argc, char **argv)
+webdb_params_t::webdb_params_t()
+ :  output_tarball_("ggcov.webdb.tgz")
 {
-    const char *file;
-    
-    argv0 = argv[0];
-    
-    popt_context = poptGetContext(PACKAGE, argc, (const char**)argv,
-    	    	    	    	  popt_options, 0);
-    poptSetOtherOptionHelp(popt_context,
-    	    	           "[OPTIONS] [executable|source|directory]...");
+}
 
-    int rc;
-    while ((rc = poptGetNextOpt(popt_context)) > 0)
-    	;
-    if (rc < -1)
-    {
-    	fprintf(stderr, "%s:%s at or near %s\n",
-	    argv[0],
-	    poptStrerror(rc),
-	    poptBadOption(popt_context, POPT_BADOPTION_NOALIAS));
-    	exit(1);
-    }
-
-    while ((file = poptGetArg(popt_context)) != 0)
-	files.append(file);
-
-    poptFreeContext(popt_context);
-
-    cov_post_args();
+webdb_params_t::~webdb_params_t()
+{
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
-#define DEBUG_GLIB 1
-#if DEBUG_GLIB
-
-static const char *
-log_level_to_str(GLogLevelFlags level)
+static logging::level_t
+log_level(GLogLevelFlags level)
 {
-    static char buf[32];
-
     switch (level & G_LOG_LEVEL_MASK)
     {
-    case G_LOG_LEVEL_ERROR: return "ERROR";
-    case G_LOG_LEVEL_CRITICAL: return "CRITICAL";
-    case G_LOG_LEVEL_WARNING: return "WARNING";
-    case G_LOG_LEVEL_MESSAGE: return "MESSAGE";
-    case G_LOG_LEVEL_INFO: return "INFO";
-    case G_LOG_LEVEL_DEBUG: return "DEBUG";
-    default:
-    	snprintf(buf, sizeof(buf), "%d", level);
-	return buf;
+    case G_LOG_LEVEL_ERROR: return logging::ERROR;
+    case G_LOG_LEVEL_CRITICAL: return logging::FATAL;
+    case G_LOG_LEVEL_WARNING: return logging::WARNING;
+    case G_LOG_LEVEL_MESSAGE: return logging::INFO;
+    case G_LOG_LEVEL_INFO: return logging::INFO;
+    case G_LOG_LEVEL_DEBUG: return logging::DEBUG;
+    default: return logging::INFO;
     }
 }
 
@@ -941,26 +869,24 @@ log_func(
     const char *msg,
     gpointer user_data)
 {
-    fprintf(stderr, "%s:%s:%s\n",
-    	(domain == 0 ? PACKAGE : domain),
-	log_level_to_str(level),
+    _log.message(log_level(level), "%s:%s:%s\n",
+	(domain == 0 ? PACKAGE : domain),
 	msg);
     if (level & G_LOG_FLAG_FATAL)
-    	exit(1);
+	exit(1);
 }
 
-#endif
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
 static int
-create_database(void)
+create_database(const webdb_params_t &params)
 {
     char *tempdir;
     char *webdb_file;
     DB *db;
     int ret;
 
-    cov_read_files(files);
+    cov_read_files(params);
 
     tempdir = g_strdup_printf("%s/ggcov-web-%d.d", get_tmpdir(), (int)getpid());
     webdb_file = g_strconcat(tempdir, "/", "ggcov.webdb", (char*)0);
@@ -970,7 +896,7 @@ create_database(void)
 
     if ((ret = db_create(&db, 0, 0)))
     {
-	fprintf(stderr, "%s: db_create(): %s\n", argv0, db_strerror(ret));
+	_log.error("db_create(): %s\n", db_strerror(ret));
 	exit(1);
     }
 
@@ -1002,8 +928,9 @@ create_database(void)
 
     create_source_symlinks(tempdir);
 
+    const char *output_tarball = params.get_output_tarball();
     const char *output_tarball_abs = (!strcmp(output_tarball, "-") ? "-" :
-			              file_make_absolute(output_tarball));
+				      file_make_absolute(output_tarball));
     systemf("cd \"%s\" ; tar -cvhzf \"%s\" *", tempdir, output_tarball_abs);
     systemf("/bin/rm -rf \"%s\"", tempdir);
 
@@ -1013,9 +940,9 @@ create_database(void)
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
 
 static int
-dump_database(const char *mode)
+dump_database(const webdb_params_t &params)
 {
-    char *webdb_file;
+    const char *webdb_file;
     DB *db;
     DBC *dbc;
     DBT key, value;
@@ -1023,13 +950,14 @@ dump_database(const char *mode)
     gboolean key_flag = FALSE;
     gboolean value_flag = FALSE;
 
-    if (files.length() != 1)
+    if (params.num_files() != 1)
     {
-	fprintf(stderr, "dump_database: must provide a .webdb filename\n");
+	_log.error("dump_database: must provide a .webdb filename\n");
 	exit(1);
     }
-    webdb_file = (char *)files.head();
+    webdb_file = params.nth_file(0);
 
+    const char *mode = params.get_dump_mode();
     for ( ; *mode ; mode++)
     {
 	switch (*mode)
@@ -1037,8 +965,8 @@ dump_database(const char *mode)
 	case 'k': key_flag = TRUE; break;
 	case 'v': value_flag = TRUE; break;
 	default:
-	    fprintf(stderr, "dump_database: argument to --dump must be a "
-			    "combination of the characters 'k','v'\n");
+	    _log.error("dump_database: argument to --dump must be a "
+		       "combination of the characters 'k','v'\n");
 	    exit(1);
 	}
     }
@@ -1046,7 +974,7 @@ dump_database(const char *mode)
 
     if ((ret = db_create(&db, 0, 0)))
     {
-	fprintf(stderr, "%s: db_create(): %s\n", argv0, db_strerror(ret));
+	_log.error("db_create(): %s\n", db_strerror(ret));
 	exit(1);
     }
 
@@ -1098,18 +1026,18 @@ dump_database(const char *mode)
 int
 main(int argc, char **argv)
 {
-#if DEBUG_GLIB
     g_log_set_handler("GLib",
-    	    	      (GLogLevelFlags)(G_LOG_LEVEL_MASK|G_LOG_FLAG_FATAL),
-    	    	      log_func, /*user_data*/0);
-#endif
+		      (GLogLevelFlags)(G_LOG_LEVEL_MASK|G_LOG_FLAG_FATAL),
+		      log_func, /*user_data*/0);
 
-    parse_args(argc, argv);
+    webdb_params_t params;
+    argparse::parser_t parser(params);
+    parser.parse(argc, argv);
 
-    if (dump_mode != NULL)
-	return dump_database(dump_mode);
+    if (params.get_dump_mode() != NULL)
+	return dump_database(params);
     else
-	return create_database();
+	return create_database(params);
 }
 
 /*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
